
/*
coolbar.exe - demonstrates IE style toolbar, standard file open dialog, status bar, RichEdit component
Copyright (C) 2000  Antti Markus (antti@pld.ttu.ee)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#define STRICT
#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <windowsx.h>
#include <commctrl.h>
#include <richedit.h>
#include <commdlg.h>
#include <string.h>
#include <stdlib.h>

#include "winclass.h"
#include "resource.h"
#include "ttwindow.h"
#ifdef __GNUC__
#include "gnu.h"
#endif

#ifndef RBBS_GRIPPERALWAYS
#define RBBS_GRIPPERALWAYS  128
#endif

//*************************************************************
//  CONSTANTS, KONSTANDID
//*************************************************************

#define MAXBUTTONS 11
#define NUMBUTTONS 3

//triistariba nuppude info
//toolbar button info
TBBUTTON tbButtons[] = {
	{STD_FILENEW,  IDM_NEW,  TBSTATE_ENABLED, TBSTYLE_BUTTON, 0L, 0},
	{STD_FILEOPEN, IDM_OPEN, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0L, 0},
	{STD_FILESAVE, IDM_SAVE, TBSTATE_ENABLED, TBSTYLE_BUTTON, 0L, 0},
};

const char* pClassName="TTWin32CoolbarEx";
const int nStatusID=200;

//*************************************************************
//  GLOBAL VARIABLES, GLOBAALSED MUUTUJAD
//*************************************************************

HINSTANCE hMainInst=0;  //instance of the application
HWND hMainWnd=0;        //phiakna handle
                        //application main window handle
HWND hRebarWnd=0;       //rebar window handle
HWND hWndToolBar=0;     //triistariba handle
                        //toolbar window handle
HWND hStatusBar=0;      //olekuriba handle
                        //status bar handle                 
HWND hEdit=0;           //richedit akna handle
                        //richedit handle                        
int cxSmIcon, cySmIcon; //vikese ikooni (reeglina 16x16) mdud
                        //small icon dimensions
int nIsToolbar=1;       //kas triistariba on nhtav?
                        //is toolbar visible?
int nCurWidth, nCurHeight;  //akna krgus ja laius
                            //width and height of the window
int nStatusH=0;         //olekuriba krgus
                        //status bar height
int nCoolH=0;           //coolbar'i krgus
                        //coolbar height                 
HINSTANCE hRichLib=0;   //riched32.dll handle
int iNT=0;              //kas programm jookseb Windows NT'l?
                        //is program running on Windows NT?

PTTWindow pRebarWnd=0;
PTTWindow pStatusbar=0;
PTTWindow pEditWnd=0;
//*************************************************************
//  FUNCTIONS, FUNKTSIOONID
//*************************************************************

//entry point for a windows program
int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR pArgs, int nHow);

//main window callback function
LRESULT CALLBACK WindowFunc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

//hook function for File Open dialog
UINT APIENTRY OFNHookProc(HWND hdlg, UINT uMsg, WPARAM wParam, LPARAM lParam);

//callback function for richedit EM_STREAMIN message
DWORD CALLBACK OpenCallback(DWORD Cookie, LPBYTE Buffer,
                            LONG BufSize, LONG* BytesRead);

//teateid ttlevad funktsioonid
//message handler functions for the main window
BOOL OnCreate(HWND hwnd, LPCREATESTRUCT lpCreateStruct);
void OnSize(HWND hwnd, UINT state, int cx, int cy);
void OnCommand(HWND hwnd, WORD id, HWND hwndCtl, UINT codeNotify);
void OnDestroy(HWND hwnd);
long OnNotify(HWND hwnd, int id, LPNMHDR pnmh);
void OnGetMinMaxInfo(HWND hwnd, LPMINMAXINFO lpMinMaxInfo);

//kuvab faili avamise dialoogi
//diplays file open dialog
void DoFileDialog(HWND hOwnerWnd);

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR pArgs, int nHow)
{
  //CS_HREDRAW | CS_VREDRAW style causes annoying flicker
  TWindowClass WinClass(/*CS_HREDRAW | CS_VREDRAW*/0,
                        WindowFunc, hInstance,
                        LoadIcon(NULL, IDI_APPLICATION),
                        LoadIcon(NULL, IDI_APPLICATION),
                        (HBRUSH)GetStockObject(WHITE_BRUSH),
                        IDR_MENU, pClassName);
  if(!WinClass.Register())
    return 0;

  hMainInst=hInstance;
                                               
  const int nMaxLen=20;                                             
  char pTitle[nMaxLen];
  LoadString(hMainInst, IDS_TITLE, pTitle, nMaxLen);
  hMainWnd=CreateWindow(pClassName, pTitle,
                        WS_OVERLAPPEDWINDOW,
                        CW_USEDEFAULT, CW_USEDEFAULT,
                        CW_USEDEFAULT, CW_USEDEFAULT,
                        NULL, NULL, hInstance, NULL);

  ShowWindow(hMainWnd, nHow);
  UpdateWindow(hMainWnd);

  MSG Message;
  while(GetMessage(&Message, NULL, 0, 0))
  {
    TranslateMessage(&Message);
    DispatchMessage(&Message);
  }
  return Message.wParam;
}

LRESULT CALLBACK WindowFunc(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
  switch(msg)
  {
    HANDLE_MSG(hwnd, WM_CREATE, OnCreate);
    HANDLE_MSG(hwnd, WM_DESTROY, OnDestroy);
    HANDLE_MSG(hwnd, WM_COMMAND, OnCommand);
    HANDLE_MSG(hwnd, WM_NOTIFY, OnNotify);
    HANDLE_MSG(hwnd, WM_GETMINMAXINFO, OnGetMinMaxInfo);
    HANDLE_MSG(hwnd, WM_SIZE, OnSize);
    default:  //all other messages are handled by default
      return DefWindowProc(hwnd, msg, wParam, lParam);
  }
  return 0;
}

BOOL OnCreate(HWND hwnd, LPCREATESTRUCT lpCreateStruct)
{
  INITCOMMONCONTROLSEX InitStruct;
  InitStruct.dwSize=sizeof(INITCOMMONCONTROLSEX);
  InitStruct.dwICC=ICC_BAR_CLASSES | ICC_COOL_CLASSES;
#ifdef __GNUC__
  //mingw32 teegid ei sisalda funktsiooni InitCommonControlsEx
  //seega tuleb kasutada LoadLibrary ja GetProcAddress funktsioone 
  //mingw32 libraries don't export InitCommonControlsEx function
  //so we have to use LoadLibrary and GetProcAddress
  char* pSystemPath=new char[MAX_PATH+20];
  GetSystemDirectory(pSystemPath, MAX_PATH);
  strcat(pSystemPath, "\\comctl32.dll");
  HMODULE hComCtlDll=LoadLibrary(pSystemPath);
  delete [] pSystemPath;
  if(!hComCtlDll)
  {
    MessageBox((HWND)NULL, "comctl32.dll loading failed", "Error", MB_OK);
    return FALSE;
  }

  typedef BOOL (*PTInitFunc)(LPINITCOMMONCONTROLSEX);
  PTInitFunc pInitCommonControlsEx=(PTInitFunc)GetProcAddress(hComCtlDll, "InitCommonControlsEx");
  if(!pInitCommonControlsEx)
  {
    MessageBox((HWND)NULL, "InitCommonControlsEx loading failed", "Error", MB_OK);
    FreeLibrary(hComCtlDll);
    return FALSE;
  }

  if(!(pInitCommonControlsEx(&InitStruct)))
  {
    MessageBox((HWND)NULL, "InitCommonControlsEx failed", "Error", MB_OK);
    FreeLibrary(hComCtlDll);
    return FALSE;
  }
  FreeLibrary(hComCtlDll);
  hComCtlDll=0;
#else
  if(!(InitCommonControlsEx(&InitStruct)))
  {
    MessageBox((HWND)NULL, "InitCommonControlsEx failed", "Error", MB_OK);
    return FALSE;
  }
#endif

  //leiame vikeste ikoonide mdud
  //get small icon dimensions
  cxSmIcon=GetSystemMetrics(SM_CXSMICON);
  cySmIcon=GetSystemMetrics(SM_CYSMICON);

  RECT rClient;
  GetClientRect(hwnd, &rClient);

  //loome rebar'i
  //create rebar
  hRebarWnd=CreateWindowEx(0, REBARCLASSNAME, NULL,
                           WS_VISIBLE | WS_BORDER | WS_CHILD | WS_CLIPCHILDREN |
                           WS_CLIPSIBLINGS | CCS_NODIVIDER | CCS_NOPARENTALIGN |
				                   RBS_VARHEIGHT | RBS_BANDBORDERS,
                           0, 0, rClient.right, 10,
				                   hwnd,
				                   (HMENU)NULL,
				                   hMainInst,
				                   (LPVOID)NULL);
  if(!hRebarWnd)
  {
    MessageBox((HWND)NULL, "Rebar creation failed", "Error", MB_OK);
    return FALSE;
  }                  
  
  pRebarWnd=new TTWindow(hRebarWnd);

  //loome triistariba
  //create toolbar
  hWndToolBar=CreateToolbarEx(hRebarWnd,
                              WS_CHILD | WS_VISIBLE | WS_CHILD | 
                              TBSTYLE_TOOLTIPS | TBSTYLE_FLAT |
                              CCS_NODIVIDER | CCS_NORESIZE | CCS_NOPARENTALIGN,
                              IDR_TOOLBAR, MAXBUTTONS, 
                              (HINSTANCE)HINST_COMMCTRL, IDB_STD_SMALL_COLOR,
                              (LPCTBBUTTON)&tbButtons, NUMBUTTONS, 
                              cxSmIcon, cySmIcon, 
                              cxSmIcon, cySmIcon, 
                              sizeof (TBBUTTON));

	if (hWndToolBar == NULL )
	{
		MessageBox ((HWND)NULL, TEXT("Toolbar not created!"), NULL, MB_OK );
    return FALSE;
	}

	REBARBANDINFO rbBand;

  //algvrtustame rebar'i ribade info
	//Initialize REBARBANDINFO for all rebar bands

	rbBand.cbSize = sizeof(REBARBANDINFO);
	rbBand.fMask = RBBIM_COLORS |	    // clrFore and clrBack are valid
		             RBBIM_CHILD |			// hwndChild is valid
		             RBBIM_CHILDSIZE |	// cxMinChild and cyMinChild are valid
		             RBBIM_STYLE |			// fStyle is valid
		             RBBIM_ID;					// wID is valid
	rbBand.clrFore = GetSysColor(COLOR_BTNTEXT);
	rbBand.clrBack = GetSysColor(COLOR_BTNFACE);

  //fStyle peaks sisaldama vrtust RBBS_GRIPPERALWAYS
  //vastasel juhul vib juhtuda, et gripperit ei kuvata
  //(oleneb comctl32.dll versioonist)
  //fStyle should include RBBS_GRIPPERALWAYS
  //otherwise gripper may or may not be displayed
  //(depends on the version number of the comctl32.dll)
	rbBand.fStyle = RBBS_NOVERT |	      // do not display in vertical orientation
		              RBBS_CHILDEDGE | 
                  RBBS_GRIPPERALWAYS; //always display gripper

	rbBand.hwndChild = hWndToolBar;
	rbBand.wID = IDR_TOOLBAR;
	rbBand.cxMinChild = rClient.right >> 1;       //0.5 x client area width
	rbBand.cyMinChild = cySmIcon+(cySmIcon >> 1); //1.5 x cySmIcon

  //lisa riba rebar'i
	// Insert band into rebar
	SendMessage(hRebarWnd, RB_INSERTBAND, (UINT) -1, (LPARAM) (LPREBARBANDINFO) &rbBand);
	
	//loome olekuriba (status bar'i)
	//let's create status bar             
	RECT rStatus;
	hStatusBar=CreateStatusWindow(WS_CHILD | WS_VISIBLE, (LPCTSTR)NULL, hwnd, nStatusID);
	SendMessage(hStatusBar, SB_SIMPLE, (WPARAM)(BOOL)TRUE, (LPARAM)0);
	pStatusbar=new TTWindow(hStatusBar);
	GetWindowRect(hStatusBar, &rStatus);
	nStatusH=rStatus.bottom-rStatus.top;
	
	//loome richedit akna
	//let's create richedit window 
	hRichLib=LoadLibraryEx("RICHED32.DLL", NULL, 0);
	if(!hRichLib)
	  MessageBox(hwnd, "Loading RICHED32.DLL failed", "Error", MB_OK | MB_ICONEXCLAMATION);
	  
	GetWindowRect(hRebarWnd, &rClient);
	nCoolH=rClient.bottom-rClient.top;               
	RECT r;
	GetClientRect(hwnd, &r);
	hEdit=CreateWindow("RichEdit", "", 
	                   WS_CHILD | WS_VISIBLE | WS_BORDER |
                     ES_MULTILINE | WS_VSCROLL | ES_AUTOVSCROLL | ES_NOHIDESEL,
                     0, nCoolH+1, r.right, 
                     r.bottom-nCoolH-nStatusH, hwnd, NULL,
                     lpCreateStruct->hInstance, NULL);
  if(!hEdit)
  {
    MessageBox(hwnd, "Richedit creation failed", "Error", MB_OK | MB_ICONEXCLAMATION);
  }                                   
  pEditWnd=new TTWindow(hEdit);
	
	GetClientRect(hwnd, &rClient);
	nCurWidth=rClient.right;
	nCurHeight=rClient.bottom;
	
	//leiame op-ssteemi versiooni
	//let's find operating system's version
  OSVERSIONINFO OSInfo;
  OSInfo.dwOSVersionInfoSize=sizeof(OSVERSIONINFO);
  GetVersionEx(&OSInfo);
  if(OSInfo.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS)
    iNT=0;
  else
    iNT=1;	

  return TRUE;
}

void OnDestroy(HWND hwnd)
{
  if(hWndToolBar)
    DestroyWindow(hWndToolBar);
  if(hRebarWnd)
  {
    DestroyWindow(hRebarWnd);
    delete pRebarWnd;
  }  
  if(hStatusBar)     
  {
    DestroyWindow(hStatusBar);
    delete pStatusbar;
  }  
  if(hEdit)           
  {
    DestroyWindow(hEdit);
    delete pEditWnd;
  }
  
  //kui me teeme FreeLibrary NT all, siis saame access violationi
  //(vhemalt koodi kirjutaja sai NT 4.0 WS SP 4 all)
  //if we do FreeLibrary under NT then we get access violation
  //(at least author of this code got it under NT 4.0 WS SP 4)
  if(!iNT && hRichLib)                                    
    FreeLibrary(hRichLib);
  PostQuitMessage(0);
}

void OnCommand(HWND hwnd, WORD id, HWND hwndCtl, UINT codeNotify)
{
  switch(id)
  {
    case IDM_SAVE:
      break;

    case IDM_NEW:
      break;

    case IDM_OPEN:
      DoFileDialog(hwnd);
      break;

    case ID_OPTIONS_SHOWTOOLBAR:
    {
      HMENU hOptionsMenu=GetMenu(hwnd);
      if(nIsToolbar)
      {
        CheckMenuItem(GetSubMenu(hOptionsMenu, 1), 0, MF_BYPOSITION | MF_UNCHECKED);
        nIsToolbar=0;
        HDWP hDefer=BeginDeferWindowPos(2); 

        hDefer=pRebarWnd->DeferHide(hDefer);  

        WINDOWPLACEMENT Placement;
        pEditWnd->GetPlacement(Placement);                                            
        int y=Placement.rcNormalPosition.top-nCoolH;
        int h=Placement.rcNormalPosition.bottom-Placement.rcNormalPosition.top+nCoolH;

        hDefer=pEditWnd->DeferMoveSize(hDefer, 0, y, 
                                       Placement.rcNormalPosition.right-Placement.rcNormalPosition.left, h);
        EndDeferWindowPos(hDefer);
      }
      else
      {
        CheckMenuItem(GetSubMenu(hOptionsMenu, 1), 0, MF_BYPOSITION | MF_CHECKED);
        nIsToolbar=1;                     
        
        HDWP hDefer=BeginDeferWindowPos(2); 
        WINDOWPLACEMENT Placement;

        pEditWnd->GetPlacement(Placement);
        int y=Placement.rcNormalPosition.top+nCoolH;
        int h=Placement.rcNormalPosition.bottom-Placement.rcNormalPosition.top-nCoolH;

        hDefer=pEditWnd->DeferMoveSize(hDefer, 0, y,
                                       Placement.rcNormalPosition.right-Placement.rcNormalPosition.left, h);
        hDefer=pRebarWnd->DeferShow(hDefer);                               
        EndDeferWindowPos(hDefer);        
      }
      break;
    }

    case IDM_EXIT:
      DestroyWindow(hwnd);
      break;
  }
}

long OnNotify(HWND hwnd, int id, LPNMHDR pnmh)
{          
  switch(pnmh->code)
  {
    //kui tooltip tahab teksti nidata
    //if tooltip wants to display text
    case TTN_NEEDTEXT:
    {
      LPTOOLTIPTEXT pttext=(LPTOOLTIPTEXT)pnmh;
      const int nMaxLen=50;
      char pText[nMaxLen];
      switch(id)
      {
        case IDM_NEW:
        {
          LoadString(hMainInst, IDS_NEWDOCTT, pText, nMaxLen);
          //pttext->lpszText="Uus dokument";
          pttext->lpszText=pText;
          break;
        }

        case IDM_OPEN:
        {                        
          LoadString(hMainInst, IDS_OPENTT, pText, nMaxLen);
          //pttext->lpszText="Avame dokumendi";
          pttext->lpszText=pText;
          break;
        }

        case IDM_SAVE:
        {
          LoadString(hMainInst, IDS_SAVETT, pText, nMaxLen);
          //pttext->lpszText="Salvestame dokumendi";
          pttext->lpszText=pText;
          break;
        }
      } //end switch(id)                          
      break;
    } //end case TTN_NEEDTEXT
  } //end switch(pnmh->code)
  return 0;
}

void OnSize(HWND hwnd, UINT state, int cx, int cy)
{

  if(state != SIZE_MINIMIZED)
  {
    HDWP hDefer;
    if(hEdit && nIsToolbar)
      hDefer=BeginDeferWindowPos(3);
    else if(hEdit && !nIsToolbar)                            
      hDefer=BeginDeferWindowPos(2);
    else
      hDefer=BeginDeferWindowPos(1);  
    WINDOWPLACEMENT Placement;
    
    RECT rPlace;
    int x, y;
    if(nIsToolbar)
    {
      pRebarWnd->GetPlacement(Placement);
      rPlace=Placement.rcNormalPosition;
      x=rPlace.right-rPlace.left+(cx-nCurWidth);
      y=rPlace.bottom-rPlace.top;  
      hDefer=pRebarWnd->DeferSize(hDefer, x, y);
    }                      
                              
    if(hEdit)
    {
      pEditWnd->GetPlacement(Placement);
      rPlace=Placement.rcNormalPosition;
      x=rPlace.right-rPlace.left+(cx-nCurWidth);
      y=rPlace.bottom-rPlace.top+(cy-nCurHeight);                     
      hDefer=pEditWnd->DeferSize(hDefer, x, y);
    }                      
    
    pStatusbar->GetPlacement(Placement);
    rPlace=Placement.rcNormalPosition;
    int w=rPlace.right-rPlace.left+(cx-nCurWidth);
    int h=rPlace.bottom-rPlace.top;
    y=rPlace.top+(cy-nCurHeight);

    hDefer=pStatusbar->DeferMoveSize(hDefer, 0, y, w, h);
    
    EndDeferWindowPos(hDefer);                          
    nCurWidth=cx;
    nCurHeight=cy;
  } //end if(state != ...
}

void OnGetMinMaxInfo(HWND hwnd, LPMINMAXINFO lpMinMaxInfo)
{
  //esimene kord kutsutakse seda funktsiooni vlja
  //enne, kui hRebarWnd mingi mistliku vrtuse saab
  //this function is first called before hRebarWnd is assigned
  //some reasonable value
  if(hRebarWnd)
  {
    //akna min krgus jtab toolbar'i, olekuriba ja redaktori jaoks piisavalt ruumi
    //window's min height leaves some room for the tool- and statusbar and richedit editor
    WORD wMinHeight=GetSystemMetrics(SM_CYMENU)+GetSystemMetrics(SM_CYMINTRACK);
    
    if(nIsToolbar)
      wMinHeight+=nCoolH;
      
    wMinHeight+=(nStatusH*3);
    if(lpMinMaxInfo->ptMinTrackSize.y < wMinHeight)
      lpMinMaxInfo->ptMinTrackSize.y = wMinHeight;
  }
}

DWORD CALLBACK OpenCallback(DWORD Cookie, LPBYTE Buffer,
                            LONG BufSize, LONG* BytesRead)
{
  ReadFile((HANDLE)Cookie, Buffer, BufSize, (LPDWORD)BytesRead, NULL);
  return 0;
}

void DoFileDialog(HWND hOwnerWnd)
{                                                       
  int nMaxLen=60;

#ifdef EST
  char pFilter[]="Tekstifailid (*.txt)\0*.txt\0Kik failid (*.*)\0*.*\0\0";
#endif
#ifdef ENG
  char pFilter[]="Text File (*.txt)\0*.txt\0All Files (*.*)\0*.*\0\0";
#endif    

  OPENFILENAME ofn;

  ofn.lStructSize=sizeof(OPENFILENAME);
  ofn.hwndOwner=hOwnerWnd;
  ofn.hInstance=hMainInst;
  ofn.lpstrFilter=pFilter;
  ofn.lpstrCustomFilter=0;
  ofn.nMaxCustFilter=0;
  ofn.nFilterIndex=0;
  ofn.nMaxFile=_MAX_PATH;
  
  ofn.lpstrFile=new char[ofn.nMaxFile];
  ofn.lpstrFile[0]='\0';
  
  ofn.lpstrFileTitle=0;
  ofn.nMaxFileTitle=_MAX_FNAME+_MAX_EXT;
  ofn.lpstrInitialDir=0;
                            
  char pTitle[nMaxLen];
  LoadString(hMainInst, IDS_OTITLE, pTitle, nMaxLen);
  ofn.lpstrTitle=pTitle;
  
  ofn.Flags=OFN_HIDEREADONLY | OFN_CREATEPROMPT | OFN_EXPLORER | OFN_ENABLEHOOK;
  ofn.nFileOffset=0;
  ofn.nFileExtension=0;
  ofn.lpstrDefExt="txt";
  ofn.lCustData=0L;
  ofn.lpfnHook=OFNHookProc;
  ofn.lpTemplateName=0;

  BOOL bRes=GetOpenFileName(&ofn);
  
  if(bRes)                         
  {    
    EDITSTREAM Stream;
    HANDLE hFile=CreateFile(ofn.lpstrFile, GENERIC_READ, 0, NULL, OPEN_EXISTING,
                            FILE_ATTRIBUTE_ARCHIVE, NULL);
    if(hFile)
    {
      Stream.dwCookie=(DWORD)hFile;
      Stream.dwError=0;
      Stream.pfnCallback=OpenCallback;
      SendMessage(pEditWnd->GetHandle(), EM_STREAMIN, SF_TEXT, (LPARAM)&Stream);
      CloseHandle(hFile);
    }                            
  }  
  delete [] ofn.lpstrFile;
}

UINT APIENTRY OFNHookProc(HWND hdlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
  switch(uMsg)
  {
    case WM_NOTIFY:
    {
      HWND hParent=GetParent(hdlg);
      OFNOTIFY *pNotifyInfo=(OFNOTIFY*)lParam;
      switch(pNotifyInfo->hdr.code)
      {
        case CDN_INITDONE:
        {
          //muudame dialoogi tekste
          //let's change dialog text's
          const int nMaxLen=20;
          char pText[nMaxLen];
          LoadString(hMainInst, IDS_OTYPE, pText, nMaxLen);
          SendMessage(hParent, CDM_SETCONTROLTEXT, (WPARAM)0x0441, (LPARAM)pText);
          LoadString(hMainInst, IDS_ONAME, pText, nMaxLen);
          SendMessage(hParent, CDM_SETCONTROLTEXT, (WPARAM)0x0442, (LPARAM)pText);
          LoadString(hMainInst, IDS_OSEARCH, pText, nMaxLen);
          SendMessage(hParent, CDM_SETCONTROLTEXT, (WPARAM)0x0443, (LPARAM)pText);
          LoadString(hMainInst, IDS_OOPEN, pText, nMaxLen);
          SendMessage(hParent, CDM_SETCONTROLTEXT, (WPARAM)IDOK, (LPARAM)pText);
          LoadString(hMainInst, IDS_OCANCEL, pText, nMaxLen);
          SendMessage(hParent, CDM_SETCONTROLTEXT, (WPARAM)IDCANCEL, (LPARAM)pText);
          return 1;
        }
        case CDN_FILEOK:                         
        {                                        
          return 0;
        }
      }
    } //end case WM_NOTIFY
  } //end switch(uMsg)
  return 0;
}


